//
// Ball.cpp
//

#include <stdlib.h> // for rand()

#include "Ball.h"

// Game engine includes.
#include "DisplayManager.h"
#include "EventCollision.h"
#include "EventOut.h"
#include "LogManager.h"
#include "WorldManager.h"

#define SPEED_FACTOR 10

// Constructor.
Ball::Ball() {

  // Set ball attributes.
  setType("ball");
  setVel();
  color =  static_cast <enum df::Color> (rand()%8);

  // Random starting position.
  df::Vector temp_pos((float) (rand()%(int)WM.getBoundary().getHorizontal()),
		      (float) (rand()%(int)WM.getBoundary().getVertical()));
  setPosition(temp_pos);

  // If want to test particles...
  //setSolidness(df::SPECTRAL);
}

// Set random speed and direction.
void Ball::setVel() {
  float x, y;
  if (rand()%2)
    x = (float) (-1.0/(rand()%SPEED_FACTOR+1));
  else
    x = (float) (1.0/(rand()%SPEED_FACTOR+1));
  if (rand()%2)
    y = (float) (-1.0/(rand()%(SPEED_FACTOR*8)+1));
  else
    y = (float) (1.0/(rand()%(SPEED_FACTOR*8)+1));
  setVelocity(df::Vector(x,y));
}

// Handle event.
int Ball::eventHandler(const df::Event *p_e) {

  if (p_e->getType() == df::COLLISION_EVENT) {
    setVel();
    return 1;
  }

  if (p_e->getType() == df::OUT_EVENT) {
    out();
    return 1;
  }

  // If we get here, we have ignored this event.
  return 0;
}

// Bounce off sides of screen.
void Ball::out(){

  // Left edge.
  if (getPosition().getX() < 0) {
    setVelocity(df::Vector(getVelocity().getX() * -1, getVelocity().getY()));
    df::Vector p(1, getPosition().getY());
    setPosition(p);
  }

  // Right edge.
  if (getPosition().getX()>WM.getBoundary().getHorizontal()-1) {
    setVelocity(df::Vector(getVelocity().getX() * -1, getVelocity().getY()));
    df::Vector p(WM.getBoundary().getHorizontal()-2, 
		 getPosition().getY());
    setPosition(p);
  }

  // Top edge.
  if (getPosition().getY() < 0) {
    setVelocity(df::Vector(getVelocity().getX(), getVelocity().getY() * -1));
    df::Vector p(getPosition().getX(), 1);
    setPosition(p);
  }
  
  // Bottom edge.
  if (getPosition().getY()>WM.getBoundary().getVertical()-1) {
    setVelocity(df::Vector(getVelocity().getX(), getVelocity().getY() * -1));
    df::Vector p(getPosition().getX(), 
		 WM.getBoundary().getVertical()-2);
    setPosition(p);
  }

}

// Draw ball on screen.
int Ball::draw() {
  return DM.drawCh(getPosition(), BALL_CHAR, color);
}
