//
// Bounce - a Dragonfly Benchmark
//
// Copyright: Mark Claypool and WPI, 2012, 2015, 2016, 2019
//

// Engine includes
#include "Config.h"
#include "Fraps.h"
#include "LogManager.h"
#include "GameManager.h"

// Game includes
#include "Bouncer.h"

#define BOUNCE_VERSION 3.4
#define BALLS_PER_STEP 1
#define ITERATION_MAX 3

int main(void) {

  // Print version info.
  printf("Bounce - a Dragonfly Benchmark (v%.1f)\n", BOUNCE_VERSION);

  LM.setLogLevel(1);
  
  // Start up GameManager.
  if (GM.startUp())  {
    LM.writeLog("Error starting game manager!");
    GM.shutDown();
    return 1;
  }

  // Particle or ball test.
  std::string s = df::Config::getInstance().match("particle");
  bool do_particles;
  if (s == "true")
    do_particles = true;
  else
    do_particles = false;
  LM.writeLog("BOUNCE: Type of test: %s",
		       do_particles ? "Particle" : "Object");

  // Number of balls per step.
  int balls_per_step = BALLS_PER_STEP;
  s = df::Config::getInstance().match("balls-per-step");
  if (s != "")
    balls_per_step = atoi(s.c_str());
  LM.writeLog("BOUNCE: Balls per step: %d", balls_per_step);

  // Number of iterations.
  int iteration_max = ITERATION_MAX;
  s = df::Config::getInstance().match("iteration-max");
  if (s != "")
    iteration_max = atoi(s.c_str());
  LM.writeLog("BOUNCE: Iteration max: %d", iteration_max);

  // Number of starting balls.
  int starting_balls = 0;
  s = df::Config::getInstance().match("starting-balls");
  if (s != "")
    starting_balls = atoi(s.c_str());
  LM.writeLog("BOUNCE: Starting balls: %d", starting_balls);

  // If non-zero, terminate after frame count instead of frame rate.
  int run_time = 0;
  s = df::Config::getInstance().match("run-time");
  if (s != "")
    run_time = atoi(s.c_str());
  if (run_time)
    LM.writeLog("BOUNCE: Run time (frames): %d", run_time);

  // Bouncer creates Balls/Particles and records stats.
  Bouncer *p_bouncer = new Bouncer(do_particles, 
				   balls_per_step,
				   starting_balls,
				   run_time,
				   iteration_max);

  // Option for Fraps.
  s = df::Config::getInstance().match("fraps");
  if (s == "true")
    new df::Fraps(); 

  // Run the game (this blocks until the game loop is over).
  GM.run();

  // Get stats.
  int bounce_mark = p_bouncer -> getBounceMark();

  // Shut game engine down.
  GM.shutDown();

  // Print statistics.
  printf("** %s bouncemark (average of %d %s): %d **\n",
	 do_particles ? "Particles" : "Objects",
	 iteration_max,
	 iteration_max > 1 ? "iterations" : "iteration",
	 bounce_mark);

  // All is well.
  return 0;
}
